
	// pic12f683

#include <pic16regs.h>

//#define CODE_PROTECT

#ifdef CODE_PROTECT

config_t __at __CONFIG0 __CONFIG = _CP_ON & _BOD_ON & _MCLRE_OFF & _PWRTE_ON & \
			    	   _CPD_ON & _INTOSCIO & _WDT_OFF & _IESO_OFF & _FCMEN_OFF;
#else

config_t __at __CONFIG0 __CONFIG = _CP_OFF & _BOD_ON & _MCLRE_OFF & _PWRTE_ON & \
			    	   _CPD_OFF & _INTOSCIO & _WDT_OFF & _IESO_OFF & _FCMEN_OFF;
#endif

#define	MCU_OSC_FREQ	8000000		// [Hz]

#include "pic12f683_extra.h"
#include "timer.h"

#define MIN_FREQ	1000		// [Hz]
#define MAX_FREQ	10000		// [Hz]

enum
  {
  SILENCE = 0b00000000,
  SOUND1  = 0b00010000,
  SOUND2  = 0b00100000,
  SOUND3  = 0b00110000
  };

#define INPUT_MASK		(SOUND1 | SOUND2 | SOUND3)

#define SOUND_OUT		GPIObits.GP2
#define SOUND_OUT_DIR		TRISIObits.TRISIO2

static unsigned short timer_value;		// Az idztbe tltend rtk.
static unsigned short timer_increment;		// A hangmagassg vltozsa.
static unsigned short timer_cycle;		// Szaggatott hang esetn a szakaszok hossza.
static unsigned short timer_counter;		// A hang hosszt szmolja.

static volatile unsigned char prev_input;
static volatile unsigned char input;		// A hang tipusa.

	/*
	    A SOUND2 tipus hang magasan kezd ezutn egyre mlyebb
	    lesz. A legmlyebb hang elrse utn enyhn hullmzani
	    kezd a hangmagassg. Ennek az llapotnak a fzisait
	    jelzi az albbi vltoz.
	*/

enum
  {
  SOUND2_UP,
  SOUND2_DOWN
  };

static volatile unsigned char sound2_state;

//******************************************************************************

	// A legrosszabb esetben 90 gpi ciklus.

static void intr(void) __interrupt 1
  {
  Dejitter_TMR1();
  PAdd16Timer1Var(timer_value);

  if (input == SOUND2)
    {
	/*
	    A hang magasbl vlik mlly. Ezutn pedig enyhn hullmz
	    magassggal szl tovbb.
	*/
    if (sound2_state == SOUND2_DOWN)
      {
	// A hang egyre mlyebb lesz.
      if (timer_value > Timer_Freq_to_Value(65535, 400, 1))
	{
	// A hang egyre mlyebb lesz addig amg el nem ri a 200 Hz-et.
	timer_value -= timer_increment;
	}
      else
	{
	// Legkzelebb nni fog.
	timer_increment = 5;
	sound2_state = SOUND2_UP;
	}
      }
    else
      {
	// A hang egyre magasabb lesz.
      if (timer_value < Timer_Freq_to_Value(65535, 500, 1))
	{
	// N a hang magassga 250 Hz-ig.
	timer_value += timer_increment;
        }
      else
	{
	// Legkzelebb cskkenni fog.
	sound2_state = SOUND2_DOWN;
	}
      }
    } // if (timer_state == SOUND2)
  else if (input == SOUND1)
    {
	// Folyamatosan hullmz magasg hang.
    if (timer_value < Timer_Freq_to_Value(65535, MAX_FREQ, 1))
      {
	// N a hang magassga 5 kHz-ig.
      timer_value += timer_increment;
      }
    else
      {
	// Ellrl kezdi a legalacsonyabb hanggal (500 Hz).
      timer_value = Timer_Freq_to_Value(65535, MIN_FREQ, 1);
      }
    }
	// A folyamatos, lland magasg hang egyszeren ll el.

	/*
	    Az idzt minden lefutsakor szintet vlt a kimenet.
	    Ez azt jelenti hogy a kimen ngyszgjel frekvencija
	    a megszakts temnek fele lesz.
	*/
  if (SOUND_OUT)
    {
    SOUND_OUT = 0;		// negatv flperidus
    }
  else
    {
    SOUND_OUT = 1;		// pozitv flperidus
    }

  AckIntTimer1();
  }

//------------------------------------------------------------------------------

	/*
	    A hangok kzl a GPIO4:5 bitekkel lehet vlasztani. A kimenet a GPIO2
	    lbon van. A program folyamatosan figyeli a hangvlaszt bemeneteket.
	*/

void main(void)
{
  INTCON = 0;
	// tkapcsol a 8 MHz-es bels oszcilltorra.
  OSCCON = (IntOscFreq_8MHz << 4) | _SCS;
	// Megvrja amg stabil lesz a rezgs.
  WaitHfOsc();

  ADCON0 = 0;			// Az A/D talakt ll.
  ANSEL = 0;			// Minden lb digitlis I/O.
  COMCON0 = 7;			// A kompartor kikapcsolva.

  GPIO = 0;
  TRISIO = 0b00111011;

  timer_cycle = 0;
  timer_counter = 0;
  timer_value = 0;
  ConfigTimer1(T1_PS_1_1 | T1_SYNC_EXT_OFF | T1_SOURCE_INT);
  EnIntTimer1();

  INTCONbits.GIE = 1;
  prev_input = SILENCE;

  while (1)
    {
	/*
	    Ha nincs vltozs akkor csak a bemenetet figyeli s nem avatkozik
	    a dolgok menetbe.
	*/
    do
      {
      input = GPIO & INPUT_MASK;
      }
    while (prev_input == input);

    prev_input = input;

    if (input == SOUND1)
      {
	// Folyamatos, hullmz magasg hang.
      timer_value = Timer_Freq_to_Value(65535, MIN_FREQ, 1);
      Write16Timer1Lit(_Timer_Freq_to_Value(65535, MIN_FREQ, 1));
      timer_increment = 5;
      SOUND_OUT_DIR = 0;
      StartTimer1();
      }
    else if (input == SOUND2)
      {
	// A hang magasrl indul, elmlyl s enyhn szirnzva szl tovbb.
      timer_value = Timer_Freq_to_Value(65535, MAX_FREQ, 1);
      Write16Timer1Lit(_Timer_Freq_to_Value(65535, MAX_FREQ, 1));
      timer_increment = 1;
      sound2_state = SOUND2_DOWN;
      SOUND_OUT_DIR = 0;
      StartTimer1();
      }
    else if (input == SOUND3)
      {
	// Folyamatos, lland magasg hang.
      timer_value = Timer_Freq_to_Value(65535, MIN_FREQ, 1);
      Write16Timer1Lit(_Timer_Freq_to_Value(65535, MIN_FREQ, 1));
      SOUND_OUT_DIR = 0;
      StartTimer1();
      }
    else
      {
      StopTimer1();
      SOUND_OUT = 0;
      }
    } // while (1)
}
